#################################################################################
#                                                                               #
#                       copyright (c) 2003 Ingeniweb SARL                       #
#                                                                               #
#################################################################################

"""GroupSpace main class"""


from Globals import InitializeClass
import string

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents, AddPortalContent, ModifyPortalContent, AddPortalFolders
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFPlone import PloneFolder
from AccessControl import Permissions, getSecurityManager, ClassSecurityInfo, Unauthorized
from OFS import Folder

from global_symbols import *


import os
import os.path
from Globals import package_home
DATA_PATH = os.path.join(package_home(globals()), 'data', 'groupspace_workflow.xml')

# GroupSpace definition
factory_type_information = {
  'id'             : 'GroupSpace',
  'portal_type'    : 'GroupSpace',
  'meta_type'      : 'GroupSpace',
  'description'    : 'GroupSpaces can be used to create workgroup places for several site members.',
  'content_icon'   : 'GroupSpace.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'GrufSpaces',
  'factory'        : 'addGroupSpace',
  'immediate_view' : 'GroupSpace_listing',                          
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    # DEPRECATED : Now use use_folder_tabs properties (See install.py)
    #{
    #'id'            : 'folderlisting',
    #'name'          : 'Contents',
    #'action'        : 'string:${folder_url}/GroupSpace_contents',
    #'permissions'   : (CMFCorePermissions.ModifyPortalContent,),
    #  'category'      : 'folder',
    #},
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${folder_url}/GroupSpace_listing',
    'permissions'  : (CMFCorePermissions.View, ),
##    'category'      : 'folder',
    },
    {
    'id'            : 'roles',
    'name'          : 'Roles',
    'action'        : 'string:${folder_url}/GroupSpace_rolesForm',
    'permissions'   : (GroupSpace_assignRolePermission,),
##    'category'      : 'folder',
    },
    {
    'id'            : 'email',
    'name'          : 'Email',
    'action'        : 'string:${folder_url}/GroupSpace_emailForm',
    'permissions'   : (GroupSpace_sendMailPermission,),
##    'category'      : 'folder',
    },
    {
    'id'            : 'local_roles',
    'name'          : 'Sharing',
    'action'        : 'string:${folder_url}/folder_localrole_form',
    'permissions'   : (GroupSpace_assignRolePermission,),
##    'category'      : 'folder',
    },
    {
    'id'           : 'edit',
    'name'         : 'Properties',
    'action'       : 'string:${folder_url}/GroupSpace_editForm',
    'permissions'  : (GroupSpace_editPermission,),
##    'category'      : 'folder',
    },
  ),
  }

# The usual factory
def addGroupSpace(self, id, title = '', REQUEST = {}):
    """
    Factory method for a GroupSpace object
    """
    obj = GroupSpace(id, title)
    self._setObject(id, obj)

    Log(LOG_DEBUG, "Adding a new GroupSpace")
    gs = getattr(self, id)

    # Hook for adding your own services in Python Scripts
    try:
        gs.add_groupspace_services()
    except:
        pass

    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')

class GroupSpace(PloneFolder.PloneFolder):
    """GroupSpace class"""
    meta_type = factory_type_information['meta_type']                   # Has not to be the same as FTI but it is the most usual case.

    manage_options = Folder.Folder.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)            # $$$ Is this clever ? Isn't it better to make the object private ?


    # Init method
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        GroupSpace.inheritedAttribute('__init__')(self,id,title)

    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(GroupSpace_editPermission, 'edit')
    def edit(self, title = '', description=''):
        """
        edit(self, title = '', description='') => object modification method
        """
        # Change title
        self.title = title
        self.description = description
        


    # This method comes from PloneFolder class.
    security.declareProtected(Permissions.access_contents_information, 'listFolderContents')
    def listFolderContents( self, spec=None, contentFilter=None, suppressHiddenFiles=0 ): 
        """
        Hook around 'contentValues' to let 'folder_contents'
        be protected.  Duplicating skip_unauthorized behavior of dtml-in.
        
        In the world of Plone we do not want to show objects that begin with a .
        So we have added a simply check.  We probably dont want to raise an
        Exception as much as we want to not show it.
        
        """

        items = self.contentValues(spec=spec, filter=contentFilter)
        l = []
        for obj in items:
            id = obj.getId()
            v = obj
            try:
                if suppressHiddenFiles and id[:1]=='.': 
                    raise Unauthorized(id, v)
                if getSecurityManager().validate(self, self, id, v):
                    l.append(obj)
            except (Unauthorized, 'Unauthorized'):
                pass
        return l



    #                                                                           #
    #                         GROUPSPACE-SPECIFIC METHODS                       #
    #                                                                           #

    # Security comments:
    #   GroupMembers adding / deletion is protected by the ManageProperties permission.
    #   Members listing is allowed upon AccessContentsInformation permission

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupSpaceMemberUsers")
    def listGroupSpaceMemberUsers(self,):
        """
        listGroupSpaceMemberUsers(self,) => List actual members, by inspecting each group's members
        """
        return self.listMemberIds()

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupSpaceVisitorUsers")
    def listGroupSpaceVisitorUsers(self,):
        """
        listGroupSpaceVisitorUsers(self,) => List actual visitors, by inspecting each group's visitors
        """
        return self.listVisitorIds()

    security.declarePrivate("listUserIdsInGroup")
    def listUserIdsInGroup(self, group_id):
        """
        listUserIdsInGroup(self, group_id) => list of strings

        XXX can be optimized !
        """
        users=self.acl_users.getUsers()
        prefix=self.acl_users.getGroupPrefix()

        avail=[]
        for user in users:
            for group in user.getGroups():
                if group_id == group or \
                       prefix+group_id == group:
                    avail.append(user.getUserName())
        return avail


    security.declareProtected(CMFCorePermissions.ManageProperties, "addGroupMember")
    def addGroupMember(self, member_id):
        """
        addGroupMember(self, member_id) => Add a group member to this group space.

        This is done by granting him the GroupMember localrole on this folder.
        """
        # Check if it is a valid member id
        # XXX TODO

        # Add its local role
        self.manage_addLocalRoles(member_id, ["GroupMember",], )


    security.declareProtected(CMFCorePermissions.ManageProperties, "removeGroupMember")
    def removeGroupMember(self, member_id):
        """
        removeGroupMember(self, member_id) => Remove a group member to this group space

        """
        # Check if it is a valid member id
        # XXX TODO

        # Remove each and every local role which starts with "Group..."
        local_roles = self.get_local_roles_for_userid(member_id)
        new_lr = []
        for lr in local_roles:
            if not lr.startswith("Group"):
                new_lr.append(lr)
        if new_lr:
            self.manage_setLocalRoles(member_id, new_lr)
        else:
            self.manage_delLocalRoles([member_id])

    security.declareProtected(GroupSpace_sendMailPermission, "getGroupById")
    def getGroupById(self, group_id):
        """
        Get group object from groupspace
        """
        
        pg = getToolByName(self, 'portal_groups')
        return pg.getGroupById(group_id)
    
    security.declareProtected(GroupSpace_sendMailPermission, "getMemberById")
    def getMemberById(self, member_id):
        """
        Get member object from groupspace
        """
        
        pm = getToolByName(self, 'portal_membership')
        return pm.getMemberById(member_id)
    
    security.declareProtected(GroupSpace_sendMailPermission, "sendMail")
    def sendMail(self, mail_text):
        """
        Send mail from groupspace
        """
        
        host = self.MailHost
        host.send(mail_text)
        
        
    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listMemberIds")
    def listMemberIds(self):
        """
        listMemberIds(self,) => Return a list of member ids for this groupspace.
        This will return both users and groups.
        """
        return self.users_with_local_role("GroupMember")

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listVisitorIds")
    def listVisitorIds(self):
        """
        listVisitorIds(self,) => Return a list of visitor ids for this groupspace.
        This will return both users and groups.
        """
        return self.users_with_local_role("GroupVisitor")

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "getMainOwner")
    def getMainOwner(self,):
        """
        getMainOwner(self,) => Return the main owner of the group space.
        It's likely that this will be the person in charge of maintaining it ! :-)
        """
        return self.listGroupOwners()[0]


    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupOwners")
    def listGroupOwners(self,):
        """
        listGroupOwners(self,) => Return a list of all persons having 'Owner' role in this group space.
        """
        ret = self.users_with_local_role("Owner")

        # XXX TODO:
        # XXX HAVE TO EXTEND THIS LIST TO ALL PEOPLE HAVING THE 'OWNER' ROLE,
        # XXX EVEN IF IT'S NOT A LOCAL ROLE !
        return ret

        


    #                                           #
    #           Various utilities               #
    #                                           #


    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "getGroupSpaceId")
    def getGroupSpaceId(self,):
        """
        getGroupSpaceId(self,) => Return groupspace's path
        """
        return string.join(self.getPhysicalPath(),'')

    security.declarePublic("canChange")
    def canChange(self,):
        """
        canChange(self,) => return true if the current user can change things in here
        """
        return self.REQUEST.AUTHENTICATED_USER.has_permission(CMFCorePermissions.ManageProperties, self)


    security.declarePublic("canSendMailToGroup")
    def canSendMailToGroup(self,):
        """
        canSendMailToGroup(self,) => return true if the current user can send a mail to the group
        """
        return self.REQUEST.AUTHENTICATED_USER.has_permission(GroupSpace_sendMailPermission, self)


    security.declareProtected(GroupSpace_sendMailPermission, "_sendMailToGroup")
    def _sendMailToGroup(self,):
        """
        Just needed to have the permission
        """
        pass

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listMembersForDisplay")
    def listMembersForDisplay(self,):
        """
        listMembersForDisplay(self,) => Same as listMemberIds() but with an accurate structure to
        display information.

        - Users are filtered
        - Roles are listed
        - Kind (user, group or unknown) is specified

        This method returns a list of tuples (massagedUsername, userType, actualUserName)
        """
        ret = []
        for usr_id in self.listMemberIds():
            obj = self.acl_users.getUser(usr_id)
            if not obj:
                # User not in the first acl_users
                ret.append((usr_id, "unknown", usr_id, ))
                continue
            if obj.isGroup():
                ret.append((obj.getUserNameWithoutGroupPrefix(), "group", usr_id, ))
            else:
                ret.append((obj.getUserName(), "user", usr_id, ))
        return ret

    security.declareProtected(ModifyPortalContent, "setInitialGroup")
    def setInitialGroup(self, group):
        # The group that we create the workspace for is automatically
        # becoming GroupMember
        self.manage_setLocalRoles(group.getId(), ['GroupMember'])

    #                                                                   #
    #           CollabSpace Interface                                   #
    #                                                                   #
    # Implementation of the CollabSpace Interface as defined in         #
    # PloneGroupSpace by Helmut Merz:                                   #
    #                                                                   #
    # http://cvs.sourceforge.net/viewcvs.py/collective/PloneGroupSpace/ # 
    #                                                                   #

    def getValidRoles(self):
        """ May be used to show a list of roles to select from in the user interface"""
        return ('GroupVisitor', 'GroupMember', 'GroupAdmin', 'GroupLeader')

    def getCollabRoles(self):
        return ('GroupVisitor', 'GroupMember', 'GroupAdmin', 'GroupLeader')

    def getCollabUsers(self, role=None):
        checkRoles = role is None and self.getCollabRoles() or [role]
        result = []
        lr = self.get_local_roles()
        for (user, roles) in lr:
            for r in roles:
                if r in checkRoles:
                    result.append(user)
                    break
        return tuple(result)

    def getCollabUserObjects(self, role=None):
        """ Returns real user objects - Member or Group - not just the ID.
        """
        userids = self.getCollabUsers(role)
        pg = getToolByName(self, 'portal_groups')
        pm = getToolByName(self, 'portal_membership')
        prefix = self.acl_users.getGroupPrefix()
        result = []
        for userid in userids:
            if userid.startswith(prefix):
                obj = pg.getGroupById(userid[len(prefix):])
            else:
                obj = pm.getMemberById(userid)
            if obj:
                result.append(obj)
        return result

    security.declareProtected(GroupSpace_assignRolePermission, 'assignToCollabRole')
    def assignToCollabRole(self, role, userid):
        if role not in self.getCollabRoles():
            raise ValueError, 'Role %s is not registered as a Collab Role here.' % role
        pm = getToolByName(self, 'portal_membership')
        member = pm.getMemberById(userid)
        if not member:
            raise ValueError, 'Member %s does not exist.' % userid
        isGid = self._isGroupId(userid)
        self.manage_addLocalRoles(userid, [role])
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberUsers'])
        self.reindexObject(idxs=['listGroupSpaceVisitorUsers'])
        self._afterAssignToCollabRole(role, userid)

    def getCollabUserDictionary(self):
        """ A convenience function to get the full information on assigned
            users with one method call. Returns a dictionary with roles as keys and
            a tuple of assigned users as the values.
        """
        result = {}
        for role in self.getCollabRoles():
            users = self.getCollabUsers(role)
            if users:
                result[role] = users
        return result

    security.declareProtected(GroupSpace_assignRolePermission, 'removeFromCollabRole')
    def removeFromCollabRole(self, role, userid):
        lr = []
        try:
            lr = list(self.get_local_roles_for_userid(userid))
        except:
            pass
        if role in lr:
            lr.remove(role)
        if lr:
            self.manage_setLocalRoles(userid, lr)
        else:
            try:
                self.manage_delLocalRoles([userid])
            except:
                pass
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberUsers'])
        self.reindexObject(idxs=['listGroupSpaceVisitorUsers'])
        self._afterRemoveFromCollabRole(role, userid)

    def _afterAssignToCollabRole(self, role, user):
        """ This method may be overridden in subclasses of GroupSpace,
            e.g. to register the assignment with the Member or Group object assigned.
        """
        pass

    def _afterRemoveFromCollabRole(self, role, user):
        """ This method may be overridden in subclasses of GroupSpace,
            e.g. to remove the assignment from the Member or Group object assigned.
        """
        pass

    def _isGroupId(self, id):
        return id.startswith(self.acl_users.getGroupPrefix())

    security.declareProtected('View', 'getCurrentGroupspace')
    def getCurrentGroupspace(self):
        """getCurrentGroupspace(self, ) => return itself
        """
        return self.restrictedTraverse('')

    #                                                                   #
    #           Extensions to the CollabSpace Interface                 #
    #                                                                   #

    security.declareProtected(ModifyPortalContent, "getMainCollabUserRole")
    def getMainCollabUserRole(self, user):
        # Return the main role of a user to be shown in the interface.
        roles= user.getRolesInContext(self)
        role = None
        if 'GroupLeader' in roles:
            role = 'GroupLeader'
        elif 'GroupAdmin'in roles:
            role = 'GroupAdmin'
        elif 'GroupMember' in roles:
            role = 'GroupMember'
        elif 'GroupVisitor' in roles:
            role = 'GroupVisitor'
        return role

# Class instanciation
InitializeClass(GroupSpace)

